"use strict";
var base_user_service_1 = require('./base-user.service');
var BaseRequestService = (function () {
    function BaseRequestService() {
        this.activeRequest = null;
        this.config = {};
        this.contentHeaders = {};
    }
    BaseRequestService.prototype.init = function (config, contentHeaders) {
        this.config = config;
        this.contentHeaders = contentHeaders;
    };
    BaseRequestService.prototype.transformUrl = function (url) {
        if (url.indexOf('://') === -1) {
            url = this.config.baseApi + url;
        }
        if (url.indexOf('access_token') > -1) {
            return url;
        }
        var accessToken = localStorage.getItem('auth_access_token');
        if (accessToken) {
            var separator = '?';
            if (url.indexOf('?') > -1) {
                separator = '&';
            }
            url += separator + "access_token=" + accessToken;
        }
        return url;
    };
    BaseRequestService.prototype.parseMessage = function (response) {
        if (response && response._body) {
            return JSON.parse(response._body);
        }
        return response;
    };
    BaseRequestService.prototype.handleUnauthorized = function (resolve, reject, self, method, callbackArgs) {
        var _this = this;
        if (!localStorage.getItem('auth_refresh_token')) {
            resolve('no_refresh_token');
            base_user_service_1.BaseUserService.redirectLogin();
            return;
        }
        this.refreshToken()
            .then(function () {
            self[method](callbackArgs)
                .then(function (response) {
                resolve(_this.parseMessage(response));
            }, function (error) {
                reject(_this.parseMessage(error));
            });
        }, function (error) {
            resolve(_this.parseMessage(error));
            base_user_service_1.BaseUserService.redirectLogin();
        });
    };
    BaseRequestService.prototype.handleRequest = function (self, method, args, originalArgs) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.activeRequest = (_a = self['http'])[method].apply(_a, args)
                .subscribe(function (response) {
                resolve(_this.parseMessage(response));
            }, function (error) {
                if (error.status === 401) {
                    _this.handleUnauthorized(resolve, reject, self, method, originalArgs);
                }
                else {
                    reject(_this.parseMessage(error));
                }
            });
            var _a;
        });
    };
    BaseRequestService.prototype.post = function (url, body) {
        if (body === void 0) { body = {}; }
        var args = [this.transformUrl(url), JSON.stringify(body), { headers: this.contentHeaders }];
        var originalArgs = [url, body];
        return this.handleRequest(this, 'post', args, originalArgs);
    };
    BaseRequestService.prototype.put = function (url, body) {
        if (body === void 0) { body = {}; }
        var args = [this.transformUrl(url), JSON.stringify(body), { headers: this.contentHeaders }];
        var originalArgs = [url, body];
        return this.handleRequest(this, 'put', args, originalArgs);
    };
    BaseRequestService.prototype.get = function (url) {
        var args = [this.transformUrl(url), { headers: this.contentHeaders }];
        var originalArgs = [url];
        return this.handleRequest(this, 'get', args, originalArgs);
    };
    BaseRequestService.prototype.delete = function (url) {
        var args = [this.transformUrl(url), { headers: this.contentHeaders }];
        var originalArgs = [url];
        return this.handleRequest(this, 'delete', args, originalArgs);
    };
    BaseRequestService.prototype.refresh = function () {
        var body = {
            client_id: this.config.client_id,
            client_secret: this.config.client_secret,
            refresh_token: localStorage.getItem('auth_refresh_token'),
            grant_type: 'refresh_token'
        };
        return this.post('/noauth/oauth/v2/token', body);
    };
    BaseRequestService.prototype.refreshToken = function () {
        if (BaseRequestService.isRefreshingToken) {
            return BaseRequestService.isRefreshingToken;
        }
        BaseRequestService.isRefreshingToken = this.refresh()
            .then(function (response) {
            localStorage.setItem('auth_access_token', response.access_token);
            localStorage.setItem('auth_expires_in', response.expires_in);
            localStorage.setItem('auth_refresh_token', response.refresh_token);
            localStorage.setItem('auth_scope', response.scope);
            localStorage.setItem('auth_token_type', response.token_type);
            BaseRequestService.isRefreshingToken = null;
        });
        return BaseRequestService.isRefreshingToken;
    };
    BaseRequestService.prototype.getPaginatedUrl = function (url, page, max) {
        if (page === 0 && max === 0) {
            return url;
        }
        var separator = '?';
        if (url.indexOf('?') > -1) {
            separator = '&';
        }
        url += separator + "page=" + page + "&page_size=" + max;
        return url;
    };
    BaseRequestService.prototype.stopRequest = function () {
        if (this.activeRequest) {
            this.activeRequest.unsubscribe();
        }
        this.activeRequest = null;
    };
    BaseRequestService.isRefreshingToken = null;
    return BaseRequestService;
}());
exports.BaseRequestService = BaseRequestService;
//# sourceMappingURL=base-request.service.js.map